function [A,b,Dc,U] = AGPS_V(f,V,lab,Th)
% Computes a matrix with prescribed residual norms and spectrum when the GMRES method is applied. 
% See the remarks after Corollary 3.7 in ,,Any Ritz value convergence behaviour is possible for the Arnoldi method and a consequence for the GMRES method" by Jurjen Duintjer
% Tebbens and Gerard Meurant
% Input:
%       f              - the n prescribed GMRES residual norms 
%       V              - the arbitrary unitary matrix (of order n)
%       lab            - the n nonzero eigenvalues of A
%       Th             - an upper triangular matrix Th of order n-1. A column is allowed to contain a zero entry above the main diagonal only if the corresponding prescribed residual 
%                        norm coincides with the previous one (stagnation).  
% Output:
%       A,b                    - The matrix and initial Arnoldi vector
%       Dc,U                   - The matrix A satisfies A = V * diag(f(1),Dc) * U^(-1) * Cn * U * diag(f(1),Dc)^(-1) * V^* where Cn is the companion matrix for lab.  
%
%     Jurjen Duintjer Tebbens, July 15, 2011 (tebbens@cs.cas.cz)
n = length(V(:,1));
if norm(V*V'-eye(n)) > 1e-12
    'Input matrix W is not unitary', return
end
if size(V) ~= [n n]
    'Input unitary matrix W has wrong dimension', return
end
Th2=zeros(n,n);
Th2(1:n-1,1:n-1)=Th;
Th2(:,n)=lab;
Th=Th2;
[U,Cn] = RitzCompTrans(Th);
h = zeros(n,1);
for j=1:n-1
    h(j) = sqrt(f(j)^2 - f(j+1)^2);
end
h(n)=f(n);
hh=h(1:n-1);
Rh = chol(eye(n-1)-hh*hh' / f(1)^2);
Rhmhh=(Rh')\hh;
J=find(Rhmhh);
c = U(1,2:n); c = c';
if length(find(c)) ~= length(J)
    'Prescribed zero Ritz values do not correspond to prescribed stagnation', return
end
dd = ones(n-1,1);
dd(J) = -Rhmhh(J)./(c(J)*f(1)^2);
Dc = diag(dd)^(-1);
b = f(1)*V(:,1);
D=diag([f(1);1./dd]);
A = V*D*(U\Cn)*U*(D\V');