function [A,b,Ds,U,H] = Corr23(sig,V,lab,Th,f1)
% Computes a matrix with prescribed Ritz values and spectrum when the Arnoldi method is applied. 
% See Corollary 2.3 in ,,Any Ritz value convergence behaviour is possible for the Arnoldi method and a consequence for the GMRES method" by Jurjen Duintjer
% Tebbens and Gerard Meurant
% Input:
%       sig            - the n-1 prescribed positive subdiagonal entries of the generated upper Hessenberg matrix  
%       V              - the arbitrary unitary matrix (of order n)
%       lab            - the n nonzero eigenvalues of A
%       Th             - an upper triangular matrix Th of order n-1. A column is allowed to contain a zero entry above the main diagonal only if the corresponding prescribed residual 
%                        norm coincides with the previous one (stagnation).  
%       f1             - the norm of the initial Arnoldi vector
% Output:
%       A,b                    - The matrix and initial Arnoldi vector
%       Ds,U                   - The matrix A satisfies A = V * Ds * U^(-1) * Cn * U * Ds^(-1) * V^* where Cn is the companion matrix for lab. 
%       H                      - The Hessenberg matrix that will be
%                                 generated by the Arnoldi process
%
%     Jurjen Duintjer Tebbens, July 15, 2011 (tebbens@cs.cas.cz)
n = length(V(:,1));
if norm(V*V'-eye(n)) > 1e-12
    'Input matrix W is not unitary', return
end
if size(V) ~= [n n]
    'Input unitary matrix W has wrong dimension', return
end
Th2=zeros(n,n);
Th2(1:n-1,1:n-1)=Th;
Th2(:,n)=lab;
Th=Th2;
[U,Cn] = RitzCompTrans(Th);
dd = ones(n,1);
for j=2:n 
    dd(j)=dd(j-1)*sig(j-1);
end
Ds = diag(dd);
b = f1*V(:,1);
H = Ds*(U\Cn)*U*Ds^(-1);
A = V*Ds*(U\Cn)*U*(Ds\V');