function [A,b,Dc,U] = Corr37(Th,f,V)
% Computes a matrix with prescribed Ritz values when the Arnoldi method is applied and with prescribed residual norms when the GMRES method is applied. 
% See Corollary 3.7 in ,,Any Ritz value convergence behaviour is possible for the Arnoldi method and a consequence for the GMRES method" by Jurjen Duintjer
% Tebbens and Gerard Meurant
% Input:
%       Th             - upper triangular matrix Th of order n whose jth
%                        column contains the prescribed Ritz values for the jth iteration. A column is allowed to contain a zero entry above the main diagonal only if the corresponding prescribed residual 
%                        norm coincides with the previous one (stagnation). The last column can contain no zero entries at all. 
%       f              - the n prescribed GMRES residual norms 
%       W              - the arbitrary unitary matrix (of order n)
% Output:
%       A,b                    - The matrix and initial Arnoldi vector
%       Dc,U                   - The matrix A satisfies A = V * diag(f(1),Dc) * U^(-1) * Cn * U * diag(f(1),Dc)^(-1) * W^*  
%
%     Jurjen Duintjer Tebbens, July 15, 2011 (tebbens@cs.cas.cz)
n = length(V(:,1));
if norm(V*V'-eye(n)) > 1e-12
    'Input matrix W is not unitary', return
end
if size(V) ~= [n n]
    'Input unitary matrix W has wrong dimension', return
end
[U,Cn] = RitzCompTrans(Th);
h = zeros(n,1);
for j=1:n-1
    h(j) = sqrt(f(j)^2 - f(j+1)^2);
end
h(n)=f(n);
hh=h(1:n-1);
Rh = chol(eye(n-1)-hh*hh'/f(1)^2);
Rhmhh=(Rh')\hh;
J=find(Rhmhh);
c = U(1,2:n); c = c';
C = U(2:n,2:n);
if length(find(c)) ~= length(J)
    'Prescribed zero Ritz values do not correspond to prescribed stagnation', return
end
dd = ones(n-1,1);
dd(J) = Rhmhh(J)./(-f(1)^2*c(J));
Dc = diag(dd)^(-1);
b = f(1)*V(:,1);
D=diag([f(1);1./dd]);
A = V*D*(U\Cn)*U*(D\V');