function [A,b,Y,Cn] = Theo33(Th,sig,W,f1)
% Computes a matrix with prescribed Ritz values when the Arnoldi method is applied. 
% See Theorem 3.3 in ,,Any Ritz value convergence behaviour is possible for the Arnoldi method and a consequence for the GMRES method" by Jurjen Duintjer
% Tebbens and Gerard Meurant
% Input:
%       Th             - upper triangular matrix Th of order n whose jth
%                        column contains the prescribed Ritz values for the jth iteration. A column is allowed to contain a zero entry above the main diagonal only if the corresponding prescribed residual 
%                        norm coincides with the previous one (stagnation). The last column can contain no zero entries at all. 
%       sig              - the n-1 prescribed  positive subdiagonal entries of the generated upper Hessenberg matrix 
%       W              - the arbitrary unitary matrix (of order n)
%       f1             - the norm of the initial Arnoldi vector
% Output:
%       A,b                    - The matrix and initial Arnoldi vector
%       Y,Cn                   - The matrix A satisfies A = W Y Cn Y^(-1) W^*  
%
%     Jurjen Duintjer Tebbens, July 15, 2011 (tebbens@cs.cas.cz)
n = length(W(:,1));
if norm(W*W'-eye(n)) > 1e-12
    'Input matrix W is not unitary', return
end
if size(W) ~= [n n]
    'Input unitary matrix W has wrong dimension', return
end
[U,Cn] = RitzCompTrans(Th);
dd = ones(n,1);
for j=2:n 
    dd(j)=dd(j-1)*sig(j-1);
end
Ds = diag(dd);
Tt = f1*Ds*U^(-1);
t=Tt(1,2:n);t=t';
T=Tt(2:n,2:n);
Tmt=(T')\t;
Rt = chol(eye(n-1)+Tmt*Tmt');
Rmt = (Rt*T)'\t;
gam = zeros(n-1,1);
for j=1:n-1
    if abs(Rmt(j)) > 1e-12
        gam(j)= Rmt(j)'/abs(Rmt(j));
    else
        gam(j) = 1;
    end
end
R = diag(gam)*Rt*T;
if max(abs(diag(R))) < 1e-6
    'Input upper triangular matrix R is close to singular', return
end
h = zeros(n,1);
h(n)=f1*sqrt(1-norm(Rmt)^2);
h(1:n-1)=f1*diag(gam)*Rmt;
b = W*h;
Y = zeros(n,n);
Y(:,1) = h(1:n);
Y(1:n-1,2:n) = R;
A = W*Y*Cn*(Y\W');