function [A,b,Y,Cn] = Theo36(Th,f,W)
% Computes a matrix with prescribed Ritz values when the Arnoldi method is applied and with prescribed residual norms when the GMRES method is applied. 
% See Theorem 3.6 in ,,Any Ritz value convergence behaviour is possible for the Arnoldi method and a consequence for the GMRES method" by Jurjen Duintjer
% Tebbens and Gerard Meurant
% Input:
%       Th             - upper triangular matrix Th of order n whose jth
%                        column contains the prescribed Ritz values for the jth iteration. A column is allowed to contain a zero entry above the main diagonal only if the corresponding prescribed residual 
%                        norm coincides with the previous one (stagnation). The last column can contain no zero entries at all. 
%       f              - the n prescribed GMRES residual norms 
%       W              - the arbitrary unitary matrix (of order n)
% Output:
%       A,b                    - The matrix and initial Arnoldi vector
%       Y,Cn                   - The matrix A satisfies A = W Y Cn Y^(-1) W^*  
%
%     Jurjen Duintjer Tebbens, July 15, 2011 (tebbens@cs.cas.cz)
n = length(W(:,1));
if norm(W*W'-eye(n)) > 1e-12
    'Input matrix W is not unitary', return
end
if size(W) ~= [n n]
    'Input unitary matrix W has wrong dimension', return
end
[U,Cn] = RitzCompTrans(Th);
h = zeros(n,1);
for j=1:n-1
    h(j) = sqrt(f(j)^2 - f(j+1)^2);
end
h(n)=f(n);
hh=h(1:n-1);
Rh = chol(eye(n-1)-hh*hh'/f(1)^2);
Rhmhh=(Rh')\hh;
J=find(Rhmhh);
c = U(1,2:n); c = c';
C = U(2:n,2:n);
if length(find(c)) ~= length(J)
    'Prescribed zero Ritz values do not correspond to prescribed stagnation', return
end
dd = ones(n-1,1);
dd(J) = -f(1)^2*Rhmhh(J)./c(J);
Dc = diag(dd);
R = Rh\(Dc'\C^(-1));
if max(abs(diag(R))) < 1e-6
    'Input upper triangular matrix R is close to singular', return
end
if size(R) ~= [n-1 n-1]
    'Input upper triangular matrix R has wrong dimension', return
end
b = W*h;
Y = zeros(n,n);
Y(:,1) = h(1:n);
Y(1:n-1,2:n) = R;
A = W*Y*Cn*(Y\W');